/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <zlib.h>



#define VER     "0.1"
#define OFFSET  0x5002
#define INSZ    32768
#define OUTSZ   65536



void std_err(void);



int main(int argc, char *argv[]) {
    FILE            *fd,
                    *fdout;
    z_stream        z;
    unsigned int   oldsz,
                    totsz;
    int             len,
                    err = 0;
    unsigned char   *in,
                    *out;


    setbuf(stdout, NULL);

    fputs("\n"
        "XIII profiles unpacker for Ubi.com account retrieving "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\nUsage: %s <file.PRO> <output_file>\n"
            "\n"
            "Note: Profile files are located in the Save folder of the game\n"
            "\n", argv[0]);
        exit(1);
    }

    printf("- open input file:    %s\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    printf("- open output file:   %s\n", argv[2]);
    fdout = fopen(argv[2], "wb");
    if(!fdout) std_err();

    printf("- seeking data offset 0x%08x\n", OFFSET);
    if(fseek(fd, OFFSET, SEEK_SET) < 0) std_err();

    fread(&totsz, 4, 1, fd);    // size of the output file, not needed here

    z.zalloc = (alloc_func)0;
    z.zfree  = (free_func)0;
    z.opaque = (voidpf)0;
    if(inflateInit(&z) != Z_OK) {
        fputs("\nError: zlib initialization error\n", stdout);
        exit(1);
    }

    in = malloc(INSZ);
    out = malloc(OUTSZ);
    if(!in || !out) std_err();

    fputs("- start unzipping:    ", stdout);
    oldsz = 0;
    while((len = fread(in, 1, INSZ, fd))) {
        fputc('.', stdout);
        z.next_in = in;
        len += z.total_in;
        while(z.total_in < len) {
            z.avail_in = 1;
            z.next_out = out;
            z.avail_out = OUTSZ;
            err = inflate(&z, Z_NO_FLUSH);
            if(err != Z_OK) break;
            fwrite(out, z.total_out - oldsz, 1, fdout);
            oldsz = z.total_out;
        }
        if(err) {
            switch(err) {
                case Z_STREAM_END:
                    fputs("\n- zip data finished\n", stdout);
                    break;
                case Z_DATA_ERROR:
                    fputs("\n- ERROR: the data in the input file is not in zip format\n", stdout);
                    break;
                case Z_NEED_DICT:
                    fputs("\n- ERROR: zlib error, you need a dictionary (option not available)\n", stdout);
                    break;
                case Z_MEM_ERROR:
                    fputs("\n- ERROR: zlib error, insufficient memory\n", stdout);
                    break;
                case Z_BUF_ERROR:
                    fputs("\n- ERROR: the output buffer for zlib uncompression is not enough\n", stdout);
                    break;
                default:
                    printf("\n- ERROR: zlib error number %d\n", err);
                    break;
            }
            break;
        }
    }

    printf("- %u bytes unzipped\n", z.total_out);
    inflateEnd(&z);
    fclose(fd);
    fclose(fdout);
    free(in);
    free(out);
    printf(
        "- Now open the output file \"%s\" and search for LogAndPass.\n"
        "  It contains the Ubi.com username and password (if stored)\n"
        "\n", argv[2]);
    return(0);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}

